<?php
/*
 * Plugin Name:       GPT WP Assistant
 * Plugin URI:        https://humancentered.dev/wordpress-chatgpt/
 * Description:       Generates text using OpenAI's GPT-3. Set the system role content in the options page, and use either [chatgpt] for a floating chatbot. Add it on a page, post, or in your template to display.
 * Version:           1.01
 * Requires at least: 5.2
 * Requires PHP:      7.2
 * Author:            Ali Jaffar
 * Author URI:        https://humancentered.dev/
 * License:           GPL v2 or later
 * License URI:       https://www.gnu.org/licenses/gpl-2.0.html
 * Update URI:        gptwpassistant
 * Text Domain:       gptwpassistant
 * Domain Path:       /languages
 */

// Register the shortcodes
add_shortcode('chatgpt', 'chatgpt_shortcode');

// Enqueue JS and CSS
add_action('wp_enqueue_scripts', 'chatgpt_enqueue_scripts');

// Register settings page
add_action('admin_menu', 'chatgpt_settings_page');

// Shortcode function
function chatgpt_shortcode() {
  $output = '';
  $output .= '<div class="chatgpt-container">';
  $output .= '<div class="chatgpt-messages"></div>';
  $output .= '<div class="chatgpt-loading"></div>';
  $output .= '<form class="chatgpt-form">';
  $output .= '<label for="message">Your message:</label>';
  $output .= '<textarea class="chatgpt-input" name="message" id="message" required></textarea>';
  $output .= '<input class="chatgpt-submit btn-custom" type="submit" value="Send">';
  $output .= '</form>';
  $output .= '</div>';

  return $output;
}

// Enqueue scripts and styles
function chatgpt_enqueue_scripts() {
  // Get the API key from the options
  $api_key = get_option('chatgpt_api_key');
  
  // Enqueue script for AJAX request
  wp_enqueue_script('chatgpt-ajax', plugin_dir_url(__FILE__) . 'gptwpassistant.js', array('jquery'), '1.0', true);
  
  // Enqueue styles
  wp_enqueue_style('chatgpt-style', plugin_dir_url(__FILE__) . 'gptwpassistant.css');
  
  // Localize the script with the API endpoint and key
    wp_localize_script( 'chatgpt-ajax', 'chatgpt_ajax_object', array(
        'api_endpoint' => get_option('chatgpt_api_endpoint'), //'https://api.openai.com/v1/chat/completions',
        'api_key' => $api_key,
        'security' => wp_create_nonce('chatgpt_ajax_nonce'),
        'ajax_url' => admin_url('admin-ajax.php')
    ));
}

// AJAX handler
add_action('wp_ajax_chatgpt_request', 'chatgpt_ajax_request');
add_action('wp_ajax_nopriv_chatgpt_request', 'chatgpt_ajax_request');
function chatgpt_ajax_request() {
  // Check the security nonce
  if (!wp_verify_nonce($_POST['security'], 'chatgpt_ajax_nonce')) {
    die('Invalid nonce');
  }
  
  // Get the input from the AJAX request
  $user_message = $_POST['message'];

  // Get the API key from the options
  $api_key = get_option('chatgpt_api_key');

  // Create the request headers
  $headers = array(
    'Content-Type' => 'application/json',
    'Authorization' => 'Bearer ' . $api_key
  );

  // Create the request body
  $model = 'gpt-3.5-turbo';

    // Add user context
    if ( wp_is_mobile() ) {
        $user_context ='using a mobile device';
    } else {
        $user_context ='using a desktop device';
    }
  $data = array(
	'model' => $model,
	'messages' => array(
		array('role' => 'system', 'content' => 'You are ChatGPT, a large language model trained by OpenAI. Answer as concisely as possible. Knowledge cutoff: July 2021. Current date is' . date("m d, Y") . '; ' . get_option('chatgpt_role') . '...you are running on a WordPress-powered website where you interact with users as a helpful assistant. Whenever a user asks to make a list, or put something in a list, inject the HTML for the unordered and ordered list directly in the response.  Note the user is '.$user_context. ' with the website URL being ' . get_site_url() . ' and the page title: ' . get_the_title()),
		array('role' => 'user', 'content' => $user_message)
	),
	"max_tokens" => 125,
	"temperature" => 0.4
  );	
	
  // Encode the request body as JSON
  $json_data = json_encode($data);

  // Create the request URL
  $url = get_option('chatgpt_api_endpoint'); //'https://api.openai.com/v1/chat/completions';

  // Send the POST request to the API endpoint
  $response = wp_remote_post($url, array(
    'headers' => $headers,
    'body' => $json_data,
    'data_format' => 'body'
  ));

  // Check if there was an error
  if (is_wp_error($response)) {
    $error_message = $response->get_error_message();
    echo json_encode(array('error' => $error_message));
  } else {
    // Get the response body
    $body = wp_remote_retrieve_body($response);

    // Decode the JSON response
    $json_response = json_decode($body, true);

    // Get the generated message from the response
    $generated_message = $json_response['choices'][0]['message']['content'];

    // Send the generated message as a response
    echo json_encode(array('message' => $generated_message));
  }

  // End the script
  die();
}

// Settings page
function chatgpt_settings_page() {
// Add the settings page to the menu
add_menu_page('ChatGPT Settings', 'ChatGPT', 'manage_options', 'chatgpt-settings', 'chatgpt_settings_page_content', 'dashicons-format-chat');

// Register the settings
add_action('admin_init', 'chatgpt_register_settings');
}

// Settings page content
function chatgpt_settings_page_content() {
?>
  <div class="wrap">
    <h1>ChatGPT Settings</h1>
    <form method="post" action="options.php">
      <?php
      settings_fields('chatgpt_options');
      do_settings_sections('chatgpt_settings');
      submit_button();
      ?>
    </form>
  </div>
  <?php
}

// Register settings
function chatgpt_register_settings() {
    // API Key setting
    register_setting('chatgpt_options', 'chatgpt_api_key', 'sanitize_text_field');

    // API Endpoint setting
    register_setting('chatgpt_options', 'chatgpt_api_endpoint', 'sanitize_text_field');

    // Role Content setting
    register_setting('chatgpt_options', 'chatgpt_role', 'sanitize_text_field');	

    // Add settings section
    add_settings_section('chatgpt_main', 'Main Settings', 'chatgpt_main_section_callback', 'chatgpt_settings');

    // Add API Key field
    add_settings_field('chatgpt_api_key', 'API Key', 'chatgpt_api_key_callback', 'chatgpt_settings', 'chatgpt_main');

    // Add API Endpoint field
    add_settings_field('chatgpt_api_endpoint', 'API Endpoint', 'chatgpt_api_endpoint_callback', 'chatgpt_settings', 'chatgpt_main');

    // Add Role Content field
    add_settings_field('chatgpt_role', 'System Role Content', 'chatgpt_role_callback', 'chatgpt_settings', 'chatgpt_main');
}

// Settings page callbacks
function chatgpt_main_section_callback() {
    echo '<p>Main settings for ChatGPT plugin.</p>';
}

function chatgpt_api_key_callback() {
    $api_key = get_option('chatgpt_api_key');
    echo '<input type="password" name="chatgpt_api_key" value="' . esc_attr($api_key) . '" />';
}

function chatgpt_api_endpoint_callback() {
    $api_endpoint = get_option('chatgpt_api_endpoint');
    if (!$api_endpoint) {
        $api_endpoint = "https://api.openai.com/v1/chat/completions";
    }
    echo '<input type="text" name="chatgpt_api_endpoint" value="' . esc_attr($api_endpoint) . '" placeholder="https://..." />';
}

function chatgpt_role_callback() {
    $api_role = get_option('chatgpt_role');
    echo '<textarea rows="5" name="chatgpt_role">' . esc_attr($api_role) . '</textarea>';
}